# -*- coding: ascii -*-
u"""
:Copyright:

 Copyright 2019 - 2025
 Andr\xe9 Malo or his licensors, as applicable

:License:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

=====================
 C ext special tests
=====================

C ext special tests.
"""
__author__ = u"Andr\xe9 Malo"

from pytest import raises

import rjsmin as _rjsmin

# pylint: disable = protected-access
py_jsmin = _rjsmin._make_jsmin(python_only=True)
py_jsmin2 = _rjsmin.jsmin_for_posers

import _rjsmin

c_jsmin = _rjsmin.jsmin

from . import _util as _test


def test_keep_bang_comments():
    """keep_bang_comments argument error"""
    with raises(RuntimeError) as e:
        c_jsmin("", keep_bang_comments=_test.badbool)
    assert e.value.args == ("yoyo",)


def test_input_type():
    """input type must be a string or bytes"""
    with raises(TypeError):
        c_jsmin(None)

    with raises(TypeError):
        py_jsmin(None)

    with raises(TypeError):
        py_jsmin2(None)



# -*- coding: ascii -*-
u"""
:Copyright:

 Copyright 2014 - 2025
 Andr\xe9 Malo or his licensors, as applicable

:License:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

================
 Test Utilities
================

Test utilities.
"""
__author__ = u"Andr\xe9 Malo"

import contextlib as _contextlib
import sys as _sys
import types as _types

from pytest import skip  # noqa pylint: disable = unused-import

try:
    import mock
except ImportError:
    from unittest import mock

try:
    reload  # pylint: disable = used-before-assignment
except NameError:
    # pylint: disable = redefined-builtin
    try:
        from importlib import reload
    except ImportError:
        from imp import reload  # noqa pylint: disable = deprecated-module

_unset = type("_unset", (object,), {})()  # pylint: disable=invalid-name


# pylint: disable = useless-object-inheritance


class Bunch(object):
    """Bunch object"""

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


@_contextlib.contextmanager
def patched_import(what, how=_unset):
    """
    Context manager to mock an import statement temporarily

    Parameters:
      what (str):
        Name of the module to mock

      how (any):
        How should it be replaced? If omitted or `unset`, a new MagicMock
        instance is created. The result is yielded as context.
    """
    # basically stolen from opensource.perlig.de/tdi/

    class_types = (type,)
    if str is bytes:
        # pylint: disable = no-member
        class_types += (_types.ClassType,)

    # pylint: disable = unnecessary-lambda-assignment
    _is_exc = lambda obj: isinstance(obj, BaseException) or (
        isinstance(obj, (type, class_types))
        and issubclass(obj, BaseException)
    )

    class FinderLoader(object):
        """Finder / Loader for meta path"""

        def __init__(self, fullname, module):
            self.module = module
            self.name = fullname
            extra = "%s." % fullname
            for key in list(_sys.modules):
                if key.startswith(extra):
                    del _sys.modules[key]
            if fullname in _sys.modules:
                del _sys.modules[fullname]

        def find_spec(self, name, path=None, target=None):
            """Find spec (Python 3.10+)"""
            # pylint: disable = unused-argument
            if name != self.name:
                return None

            from importlib import util

            return util.spec_from_loader(name, self)

        def find_module(self, fullname, path=None):
            """Find the module"""
            # pylint: disable = unused-argument
            if fullname == self.name:
                return self
            return None

        def load_module(self, fullname):
            """Load the module"""
            if _is_exc(self.module):
                raise self.module
            _sys.modules[fullname] = self.module
            return self.module

        def create_module(self, spec):
            """Create module"""
            if _is_exc(self.module):
                raise self.module
            return self.module

        def exec_module(self, module):
            """Execute module"""

    realmodules = _sys.modules.copy()
    try:
        obj = FinderLoader(what, mock.MagicMock() if how is _unset else how)
        realpath = _sys.meta_path[:]
        try:
            _sys.meta_path[:] = [obj] + realpath
            old, parts = _unset, what.rsplit(".", 1)
            if len(parts) == 2:
                parent, base = parts[0], parts[1]
                if parent in _sys.modules:
                    parent = _sys.modules[parent]
                    if hasattr(parent, base):
                        old = getattr(parent, base)
                        setattr(parent, base, obj.module)
            try:
                yield obj.module
            finally:
                if old is not _unset:
                    setattr(parent, base, old)
        finally:
            _sys.meta_path[:] = realpath
    finally:
        _sys.modules.clear()
        _sys.modules.update(realmodules)


def uni(value):
    """
    Create unicode from raw string with unicode escapes

    Parameters:
      value (str):
        String, which encodes to ascii and decodes as unicode_escape

    Returns:
      str: The decoded string
    """
    return value.encode("ascii").decode("unicode_escape")


class badstr(object):  # pylint: disable = invalid-name
    """bad string"""

    def __str__(self):
        raise RuntimeError("yo")


badstr = badstr()


class badbytes(object):  # pylint: disable = invalid-name
    """bad bytes"""

    def __bytes__(self):
        raise RuntimeError("yoyo")

    if str is bytes:
        __str__ = __bytes__


badbytes = badbytes()


class badbool(object):  # pylint: disable = invalid-name
    """bad bool"""

    def __bool__(self):
        raise RuntimeError("yoyo")

    if str is bytes:
        __nonzero__ = __bool__


badbool = badbool()


class baditer(object):  # pylint: disable = invalid-name
    """bad iter"""

    def __init__(self, *what):
        self._what = iter(what)

    def __iter__(self):
        return self

    def __next__(self):
        for item in self._what:
            if isinstance(item, Exception):
                raise item
            return item

    next = __next__


def calls(value):
    """
    Map calls to list of tuples

    Parameters:
      value (Mock):
        The mock object to inspect

    Returns:
      list: The List of call tuples
    """
    return list(map(tuple, value.mock_calls))



# This code is original from jsmin by Douglas Crockford, it was translated to
# Python by Baruch Even. It was rewritten by Dave St.Germain for speed.
#
# The MIT License (MIT)
# 
# Copyright (c) 2013 Dave St.Germain
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import sys
is_3 = sys.version_info >= (3, 0)
if is_3:
    import io
else:
    import StringIO
    try:
        import cStringIO
    except ImportError:
        cStringIO = None


__all__ = ['jsmin', 'JavascriptMinify']
__version__ = '2.0.9'


def jsmin(js):
    """
    returns a minified version of the javascript string
    """
    if not is_3:        
        if cStringIO and not isinstance(js, unicode):
            # strings can use cStringIO for a 3x performance
            # improvement, but unicode (in python2) cannot
            klass = cStringIO.StringIO
        else:
            klass = StringIO.StringIO
    else:
        klass = io.StringIO
    ins = klass(js)
    outs = klass()
    JavascriptMinify(ins, outs).minify()
    return outs.getvalue()


class JavascriptMinify(object):
    """
    Minify an input stream of javascript, writing
    to an output stream
    """

    def __init__(self, instream=None, outstream=None):
        self.ins = instream
        self.outs = outstream

    def minify(self, instream=None, outstream=None):
        if instream and outstream:
            self.ins, self.outs = instream, outstream
        
        self.is_return = False
        self.return_buf = ''
        
        def write(char):
            # all of this is to support literal regular expressions.
            # sigh
            if char in 'return':
                self.return_buf += char
                self.is_return = self.return_buf == 'return'
            self.outs.write(char)
            if self.is_return:
                self.return_buf = ''

        read = self.ins.read

        space_strings = "abcdefghijklmnopqrstuvwxyz"\
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_$\\"
        starters, enders = '{[(+-', '}])+-"\''
        newlinestart_strings = starters + space_strings
        newlineend_strings = enders + space_strings
        do_newline = False
        do_space = False
        escape_slash_count = 0
        doing_single_comment = False
        previous_before_comment = ''
        doing_multi_comment = False
        in_re = False
        in_quote = ''
        quote_buf = []
        
        previous = read(1)
        if previous == '\\':
            escape_slash_count += 1
        next1 = read(1)
        if previous == '/':
            if next1 == '/':
                doing_single_comment = True
            elif next1 == '*':
                doing_multi_comment = True
                previous = next1
                next1 = read(1)
            else:
                write(previous)
        elif not previous:
            return
        elif previous >= '!':
            if previous in "'\"":
                in_quote = previous
            write(previous)
            previous_non_space = previous
        else:
            previous_non_space = ' '
        if not next1:
            return

        while 1:
            next2 = read(1)
            if not next2:
                last = next1.strip()
                if not (doing_single_comment or doing_multi_comment)\
                    and last not in ('', '/'):
                    if in_quote:
                        write(''.join(quote_buf))
                    write(last)
                break
            if doing_multi_comment:
                if next1 == '*' and next2 == '/':
                    doing_multi_comment = False
                    next2 = read(1)
            elif doing_single_comment:
                if next1 in '\r\n':
                    doing_single_comment = False
                    while next2 in '\r\n':
                        next2 = read(1)
                        if not next2:
                            break
                    if previous_before_comment in ')}]':
                        do_newline = True
                    elif previous_before_comment in space_strings:
                        write('\n')
            elif in_quote:
                quote_buf.append(next1)

                if next1 == in_quote:
                    numslashes = 0
                    for c in reversed(quote_buf[:-1]):
                        if c != '\\':
                            break
                        else:
                            numslashes += 1
                    if numslashes % 2 == 0:
                        in_quote = ''
                        write(''.join(quote_buf))
            elif next1 in '\r\n':
                if previous_non_space in newlineend_strings \
                    or previous_non_space > '~':
                    while 1:
                        if next2 < '!':
                            next2 = read(1)
                            if not next2:
                                break
                        else:
                            if next2 in newlinestart_strings \
                                or next2 > '~' or next2 == '/':
                                do_newline = True
                            break
            elif next1 < '!' and not in_re:
                if (previous_non_space in space_strings \
                    or previous_non_space > '~') \
                    and (next2 in space_strings or next2 > '~'):
                    do_space = True
                elif previous_non_space in '-+' and next2 == previous_non_space:
                    # protect against + ++ or - -- sequences
                    do_space = True
                elif self.is_return and next2 == '/':
                    # returning a regex...
                    write(' ')
            elif next1 == '/':
                if do_space:
                    write(' ')
                if in_re:
                    if previous != '\\' or (not escape_slash_count % 2) or next2 in 'gimy':
                        in_re = False
                    write('/')
                elif next2 == '/':                    
                    doing_single_comment = True
                    previous_before_comment = previous_non_space
                elif next2 == '*':
                    doing_multi_comment = True
                    previous = next1
                    next1 = next2
                    next2 = read(1)
                else:
                    in_re = previous_non_space in '(,=:[?!&|' or self.is_return # literal regular expression
                    write('/')
            else:
                if do_space:
                    do_space = False
                    write(' ')
                if do_newline:
                    write('\n')
                    do_newline = False
                    
                write(next1)
                if not in_re and next1 in "'\"":
                    in_quote = next1
                    quote_buf = []

            previous = next1
            next1 = next2

            if previous >= '!':
                previous_non_space = previous

            if previous == '\\':
                escape_slash_count += 1
            else:
                escape_slash_count = 0



#!/usr/bin/env python
# -*- coding: ascii -*-
u"""
:Copyright:

 Copyright 2011 - 2025
 Andr\xe9 Malo or his licensors, as applicable

:License:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

===========================================
 rJSmin - A Javascript Minifier For Python
===========================================

rJSmin - A Javascript Minifier For Python.
"""
from __future__ import print_function
__author__ = u"Andr\xe9 Malo"

import os as _os
import posixpath as _posixpath
import sys as _sys

# pylint: disable = no-name-in-module, import-error, raise-missing-from
import setuptools as _setuptools

# pylint: disable = invalid-name


def _doc(filename):
    """Read docs file"""
    # pylint: disable = unspecified-encoding
    args = {} if str is bytes else dict(encoding="utf-8")
    try:
        with open(_os.path.join("docs", filename), **args) as fp:
            return fp.read()
    except IOError:
        return None


package = dict(
    name="rjsmin",
    top=".",
    pathname=".",
    desc="Javascript Minifier",
    longdesc=_doc("DESCRIPTION"),
    author=__author__,
    email="nd@perlig.de",
    url="https://opensource.perlig.de/rjsmin/",
    license="Apache License, Version 2.0",
    license_files=["LICENSE"],

    packages=False,
    py_modules=["rjsmin"],
    version_file="rjsmin.py",
    install_requires=[],

    entry_points={},

    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved",
        "Operating System :: OS Independent",
        "Programming Language :: C",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: Jython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Filters",
        "Topic :: Utilities",
    ],
)


class BuildFailed(Exception):
    """ The build has failed """


from distutils.command import build_ext as _build_ext  # pylint: disable = wrong-import-order
from distutils import errors as _errors  # pylint: disable = wrong-import-order
class build_ext(_build_ext.build_ext):  # pylint: disable = no-init
    """Improved extension building code"""

    def run(self):
        """Unify exception"""
        try:
            _build_ext.build_ext.run(self)
        except _errors.DistutilsPlatformError:
            raise BuildFailed()

    def build_extension(self, ext):
        """
        Build C extension - with extended functionality

        The following features are added here:

        - The macros ``EXT_PACKAGE`` and ``EXT_MODULE`` will be filled (or
          unset) depending on the extensions name, but only if they are not
          already defined.

        - "." is added to the include directories (for cext.h)

        :Parameters:
          `ext` : `Extension`
            The extension to build

        :Return: whatever ``distutils.command.build_ext.build_ext`` returns
        :Rtype: any
        """
        # handle name macros
        macros = dict(ext.define_macros or ())
        tup = ext.name.split(".")
        if len(tup) == 1:
            pkg, mod = None, tup[0]
        else:
            pkg, mod = ".".join(tup[:-1]), tup[-1]
        if pkg is not None and "EXT_PACKAGE" not in macros:
            ext.define_macros.append(("EXT_PACKAGE", pkg))
        if "EXT_MODULE" not in macros:
            ext.define_macros.append(("EXT_MODULE", mod))
        if pkg is None:
            macros = dict(ext.undef_macros or ())
            if "EXT_PACKAGE" not in macros:
                ext.undef_macros.append("EXT_PACKAGE")

        try:
            return _build_ext.build_ext.build_extension(self, ext)
        except (
            _errors.CCompilerError,
            _errors.DistutilsExecError,
            _errors.DistutilsPlatformError,
            IOError,
            ValueError,
        ) as e:
            raise BuildFailed(str(e))


class Extension(_setuptools.Extension):
    """improved functionality"""

    def __init__(self, *args, **kwargs):
        """Initialization"""
        version = kwargs.pop("version")
        self.depends = []
        if "depends" in kwargs:
            self.depends = kwargs["depends"]
        _setuptools.Extension.__init__(self, *args, **kwargs)
        self.define_macros.append(("EXT_VERSION", version))

        # add include path
        included = "."
        if included not in self.include_dirs:
            self.include_dirs.append(included)

        # add cext.h to the dependencies
        cext_h = _posixpath.normpath(_posixpath.join(included, "cext.h"))
        for item in self.depends:
            if _posixpath.normpath(item) == cext_h:
                break
        else:
            self.depends.append(cext_h)


EXTENSIONS = lambda v: [Extension("_rjsmin", ["rjsmin.c"], version=v)]


def do_setup(cext):
    """Main"""
    # pylint: disable = too-many-branches
    # pylint: disable = unspecified-encoding

    args = {} if str is bytes else dict(encoding="utf-8")
    version_file = "%s/%s" % (
        package["pathname"],
        package.get("version_file", "__init__.py"),
    )
    with open(version_file, **args) as fp:
        for line in fp:  # pylint: disable = redefined-outer-name
            if line.startswith("__version__"):
                version = line.split("=", 1)[1].strip()
                if version.startswith(("'", '"')):
                    version = version[1:-1].strip()
                break
        else:
            raise RuntimeError("Version not found")

    kwargs = {}

    if not cext or "java" in _sys.platform.lower():
        extensions = []
    else:
        extensions = EXTENSIONS(version)

    if extensions:
        if "build_ext" in globals():
            kwargs.setdefault("cmdclass", {})["build_ext"] = build_ext
        kwargs["ext_modules"] = extensions

        cflags = None
        if _os.environ.get("CFLAGS") is None:
            from distutils import ccompiler as _ccompiler

            compiler = _ccompiler.get_default_compiler()
            try:
                with open("debug.%s.cflags" % compiler) as fp:
                    cflags = (
                        " ".join(
                            [
                                line
                                for line in (line.strip() for line in fp)
                                if line and not line.startswith("#")
                            ]
                        ).split()
                        or None
                    )
            except IOError:
                pass

        if cflags:
            gcov = "coverage" in " ".join(cflags)
            for ext in extensions:
                # pylint: disable = attribute-defined-outside-init
                ext.extra_compile_args = (
                    getattr(ext, "extra_compile_args", []) + cflags
                )
                if gcov:
                    ext.libraries.append("gcov")

    if package.get("packages", True):
        kwargs["packages"] = [package["top"]] + [
            "%s.%s" % (package["top"], item)
            for item in _setuptools.find_packages(package["pathname"])
        ]

    if package.get("py_modules"):
        kwargs["py_modules"] = package["py_modules"]
    if package.get("license_files"):
        kwargs["license_files"] = package["license_files"]
    if package.get("license"):
        kwargs["license"] = package["license"]
    if package.get("classifiers"):
        kwargs["classifiers"] = package["classifiers"]
    if package.get("entry_points"):
        kwargs["entry_points"] = package["entry_points"]

    _setuptools.setup(
        name=package["name"],
        author=package["author"],
        author_email=package["email"],
        description=package["desc"],
        long_description=package["longdesc"],
        url=package["url"],
        install_requires=package["install_requires"],
        version=version,
        zip_safe=False,
        **kwargs
    )


def setup():
    """Run setup"""
    try:
        do_setup(True)
    except BuildFailed:
        env = "SETUP_CEXT_REQUIRED"
        if _os.environ.get(env, "") not in ("", "0"):
            raise
        print(
            "C extension build failed - building python only version now. "
            "Set '%s' environment variable to '1' to make it fail." % (env,),
            file=_sys.stderr,
        )
        do_setup(False)


if __name__ == "__main__":
    setup()



# -*- coding: ascii -*-
u"""
:Copyright:

 Copyright 2019 - 2025
 Andr\xe9 Malo or his licensors, as applicable

:License:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

========
 Basics
========

test for basics.
"""
__author__ = u"Andr\xe9 Malo"

import os as _os

import rjsmin as _rjsmin

# pylint: disable = protected-access
py_jsmin = _rjsmin._make_jsmin(python_only=True)
py_jsmin2 = _rjsmin.jsmin_for_posers

import _rjsmin

c_jsmin = _rjsmin.jsmin


def load(name):
    """Load a file"""
    with open(_os.path.join(_os.path.dirname(__file__), name), "rb") as fp:
        return fp.read()


def save(name, value):
    """Load a file"""
    with open(_os.path.join(_os.path.dirname(__file__), name), "wb") as fp:
        fp.write(value)


def test_string():
    """Test incomplete string"""
    inp = load("js/incomplete_string.js")
    exp = load("js/incomplete_string.min.js")
    # save('js/incomplete_string.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_regex():
    """Test incomplete regex"""
    inp = load("js/incomplete_regex.js")
    exp = load("js/incomplete_regex.min.js")
    # save('js/incomplete_regex.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_regex2():
    """Test incomplete regex"""
    inp = load("js/incomplete_regex2.js")
    exp = load("js/incomplete_regex2.min.js")
    # save('js/incomplete_regex2.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_comment():
    """Test incomplete comment"""
    inp = load("js/incomplete_comment.js")
    exp = load("js/incomplete_comment.min.js")
    # save('js/incomplete_comment.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_comment2():
    """Test incomplete comment"""
    inp = load("js/incomplete_comment2.js")
    exp = load("js/incomplete_comment2.min.js")
    # save('js/incomplete_comment2.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_comment_bang():
    """Test incomplete bang comment"""
    inp = load("js/incomplete_bang_comment.js")
    exp = load("js/incomplete_bang_comment.min.js")
    # save('js/incomplete_bang_comment.min.js',
    #      py_jsmin(inp, keep_bang_comments=True))
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp


def test_comment_bang2():
    """Test incomplete bang comment"""
    inp = load("js/incomplete_bang_comment2.js")
    exp = load("js/incomplete_bang_comment2.min.js")
    # save('js/incomplete_bang_comment2.min.js',
    #      py_jsmin(inp, keep_bang_comments=True))
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp


def test_slash():
    """Test lone slash"""
    inp = load("js/lone_slash.js")
    exp = load("js/lone_slash.min.js")
    # save('js/lone_slash.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


# def test_basic_bang():
#     """ Test basic.js with bang comments """
#     inp = load('js/basic.js')
#     exp = load('js/basic.bang.js')
#     # save('js/basic.bang.js', py_jsmin(inp, keep_bang_comments=True))
#     assert py_jsmin(inp, keep_bang_comments=True) == exp
#     assert py_jsmin2(inp, keep_bang_comments=True) == exp
#     assert c_jsmin(inp, keep_bang_comments=True) == exp
#
#     inp = inp.decode('latin-1')
#     exp = exp.decode('latin-1')
#     assert py_jsmin(inp, keep_bang_comments=True) == exp
#     assert py_jsmin2(inp, keep_bang_comments=True) == exp
#     assert c_jsmin(inp, keep_bang_comments=True) == exp



# -*- coding: ascii -*-
u"""
:Copyright:

 Copyright 2019 - 2025
 Andr\xe9 Malo or his licensors, as applicable

:License:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

=========
 Issue 8
=========

Template String support.
"""
__author__ = u"Andr\xe9 Malo"

import os as _os

import rjsmin as _rjsmin

# pylint: disable = protected-access
py_jsmin = _rjsmin._make_jsmin(python_only=True)
py_jsmin2 = _rjsmin.jsmin_for_posers

import _rjsmin

c_jsmin = _rjsmin.jsmin


def load(name):
    """Load a file"""
    with open(_os.path.join(_os.path.dirname(__file__), name), "rb") as fp:
        return fp.read()


def save(name, value):
    """Load a file"""
    with open(_os.path.join(_os.path.dirname(__file__), name), "wb") as fp:
        fp.write(value)


def test_issue():
    """Test issue"""
    inp = load("js/issue8.js")
    exp = load("js/issue8.min.js")
    # save('js/issue8.min.js', py_jsmin(inp))
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp) == exp
    assert py_jsmin2(inp) == exp
    assert c_jsmin(inp) == exp


def test_issue_bang():
    """Test issue with bang comments"""
    inp = load("js/issue8.js")
    exp = load("js/issue8.bang.js")
    # save('js/issue8.bang.js', py_jsmin(inp, keep_bang_comments=True))
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp

    inp = inp.decode("latin-1")
    exp = exp.decode("latin-1")
    assert py_jsmin(inp, keep_bang_comments=True) == exp
    assert py_jsmin2(inp, keep_bang_comments=True) == exp
    assert c_jsmin(inp, keep_bang_comments=True) == exp
